describe('scrolling target with offset into view', function (t) {
    
    t.describe('Scroll target into view should work', function (t) {
        var scrollable;
        var inner;
    
        t.beforeEach(function() {
            document.body.innerHTML = 
                '<div id="scrollable" style="background:#000;width:200px;height:100px;overflow:scroll">' +
                    '<div id="inner" style="width:2000px;height:400px;background:#F00">INNER</div>' +
                '</div>';
    
            scrollable  = t.query('#scrollable')[ 0 ];
            inner       = t.query('#inner')[ 0 ];
        })
    
        t.it('Should be able to click on target with offset supplied', function(t) {
            t.firesOnce(inner, 'click');
            
            inner.addEventListener('click', function (e) {
                // in IE 10 Y offset can be 200.5
                t.isApprox(e.offsetX, 400, 1, "Correct X offset")
                t.isApprox(e.offsetY, 200, 1, "Correct Y offset")
            })
            
            t.ok(t.elementIsScrolledOut(inner, [ 400, 200 ]), 'Click point is initially scrolled out')
    
            t.chain(
                { waitFor : 100 },
                
                { click : '#inner', offset : [ 400, 200 ] },
                
                function() {
                    t.expect(scrollable.scrollLeft).toBe(399)
                    t.expect(scrollable.scrollTop).toBe(199)
                }                
            )
        })
    })

    
    t.describe('Scroll target into view should work #2', function (t) {
        var scrollable;
        var inner;
    
        t.beforeEach(function() {
            document.body.innerHTML = 
                '<div id="scrollable" style="background:#888;width:200px;height:100px;overflow:scroll">' +
                    'Some text<br><br><br>' +
                    '<div id="wrapper">' +
                        'Some text<br><br><br>' +
                        '<div id="inner" style="width:2000px;height:400px;background:#F00">INNER</div>' +
                    '</div>' +
                '</div>';
    
            scrollable  = t.query('#scrollable')[ 0 ];
            inner       = t.query('#inner')[ 0 ];
        })
    
        t.it('Should be able to click on target with offset supplied', function(t) {
            t.firesOnce(inner, 'click');
            
            inner.addEventListener('click', function (e) {
                // in IE 10 Y offset can be 200.5
                t.isApprox(e.offsetX, 400, 1, "Correct X offset")
                t.isApprox(e.offsetY, 200, 1, "Correct Y offset")
            })
    
            t.chain(
                { waitFor : 100 },
                { click : '#inner', offset : [ 400, 200 ] },
    
                function () {
                }
            )
        })
    })
    
});

